import os
import re
from datetime import datetime, timedelta

from django.core.files import File
from django.core.files.base import ContentFile
from django.core.files.storage import default_storage
from django.template.loader import render_to_string
from django.test import TransactionTestCase, override_settings
from django.utils import timezone

import freezegun
import pytest

from aleksis.core.models import PDFFile, Person
from aleksis.core.util.pdf import clean_up_expired_pdf_files

pytestmark = pytest.mark.django_db


@pytest.mark.skip
@pytest.mark.usefixtures("celery_worker")
@override_settings(CELERY_BROKER_URL="memory://localhost//")
class PDFFIleTest(TransactionTestCase):
    serialized_rollback = True

    _test_pdf = os.path.join(os.path.dirname(os.path.abspath(__file__)), "test.pdf")

    def _get_test_html(self):
        return ContentFile(render_to_string("core/pages/test_pdf.html"), name="source.html")

    def test_pdf_file(self):
        dummy_person = Person.objects.create(first_name="Jane", last_name="Doe")

        html = self._get_test_html()
        file_object = PDFFile.objects.create(person=dummy_person, html_file=html)
        assert isinstance(file_object.expires_at, datetime)
        assert file_object.expires_at > timezone.now()
        assert not bool(file_object.file)

        with open(self._test_pdf, "rb") as f:
            file_object.file.save("print.pdf", File(f))
        file_object.save()
        re_base = r"pdfs/print_[a-zA-Z0-9]+\.pdf"
        assert re.match(re_base, file_object.file.name)

    def test_delete_signal(self):
        dummy_person = Person.objects.create(first_name="Jane", last_name="Doe")
        file_object = PDFFile.objects.create(person=dummy_person, html_file=self._get_test_html())
        with open(self._test_pdf, "rb") as f:
            file_object.file.save("print.pdf", File(f))
        file_object.save()

        file_path = file_object.file.path

        assert default_storage.exists(file_path)
        file_object.delete()
        assert not default_storage.exists(file_path)

    def test_delete_expired_files(self):
        # Create test instances
        dummy_person = Person.objects.create(first_name="Jane", last_name="Doe")
        file_object = PDFFile.objects.create(person=dummy_person, html_file=self._get_test_html())
        file_object2 = PDFFile.objects.create(
            person=dummy_person,
            html_file=self._get_test_html(),
            expires_at=timezone.now() + timedelta(minutes=10),
        )
        with open(self._test_pdf, "rb") as f:
            file_object.file.save("print.pdf", File(f))
            file_object2.file.save("print.pdf", File(f))
        file_object.save()
        file_object2.save()

        clean_up_expired_pdf_files()
        assert PDFFile.objects.get(pk=file_object.pk)
        assert PDFFile.objects.get(pk=file_object2.pk)

        # Prepare times
        test_time_before = timezone.now() + timedelta(minutes=2.5)
        test_time_between = timezone.now() + timedelta(minutes=4)
        test_time_after = timezone.now() + timedelta(minutes=15)

        # None of the files are expired
        with freezegun.freeze_time(test_time_before):
            clean_up_expired_pdf_files()
            assert PDFFile.objects.get(pk=file_object.pk)
            assert PDFFile.objects.get(pk=file_object2.pk)

        # One file is expired
        with freezegun.freeze_time(test_time_between):
            clean_up_expired_pdf_files()
            with pytest.raises(PDFFile.DoesNotExist):
                PDFFile.objects.get(pk=file_object.pk)
            assert PDFFile.objects.get(pk=file_object2.pk)

        # Both files are expired
        with freezegun.freeze_time(test_time_after):
            clean_up_expired_pdf_files()
            with pytest.raises(PDFFile.DoesNotExist):
                PDFFile.objects.get(pk=file_object.pk)
            with pytest.raises(PDFFile.DoesNotExist):
                PDFFile.objects.get(pk=file_object2.pk)



# Generated by Django 3.2.10 on 2021-12-15 21:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0027_person_place_of_birth'),
    ]

    operations = [
        migrations.AlterField(
            model_name='oauthapplication',
            name='authorization_grant_type',
            field=models.CharField(blank=True, choices=[('authorization-code', 'Authorization code'), ('implicit', 'Implicit'), ('password', 'Resource owner password-based'), ('client-credentials', 'Client credentials'), ('openid-hybrid', 'OpenID connect hybrid')], default='', max_length=32),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='person',
            name='place_of_birth',
            field=models.CharField(blank=True, default='', max_length=255, verbose_name='Place of birth'),
            preserve_default=False,
        ),
    ]



# Generated by Django 3.2.12 on 2022-02-18 21:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0035_preference_model_unique'),
    ]

    operations = [
        migrations.AddField(
            model_name='additionalfield',
            name='help_text',
            field=models.TextField(blank=True, verbose_name='Help text / description'),
        ),
        migrations.AddField(
            model_name='additionalfield',
            name='required',
            field=models.BooleanField(default=False, verbose_name='Required'),
        ),
    ]



import graphene


class SearchModelType(graphene.ObjectType):
    absolute_url = graphene.String()
    name = graphene.String()
    icon = graphene.String()

    def resolve_absolute_url(root, info, **kwargs):
        if hasattr(root, "get_absolute_url"):
            return root.get_absolute_url()
        else:
            return "#!"

    def resolve_name(root, info, **kwargs):
        return str(root)

    def resolve_icon(root, info, **kwargs):
        return getattr(root, "icon_", "")


class SearchResultType(graphene.ObjectType):
    app_label = graphene.String()
    model_name = graphene.String()
    score = graphene.Int()
    obj = graphene.Field(SearchModelType)
    verbose_name = graphene.String()
    verbose_name_plural = graphene.String()
    text = graphene.String()

    def resolve_obj(root, info, **kwargs):  # noqa
        return root.object



# Generated by Django 3.2.8 on 2021-11-04 19:35

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('core', '0024_oauth_grant_types_optional'),
    ]

    operations = [
        migrations.AlterField(
            model_name='oauthaccesstoken',
            name='user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='core_oauthaccesstoken', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='oauthapplication',
            name='user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='core_oauthapplication', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='oauthgrant',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='core_oauthgrant', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='oauthidtoken',
            name='user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='core_oauthidtoken', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='oauthrefreshtoken',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='core_oauthrefreshtoken', to=settings.AUTH_USER_MODEL),
        ),
    ]



from django.core.exceptions import PermissionDenied

import graphene
from graphene_django import DjangoObjectType

from ..models import Notification


class NotificationType(DjangoObjectType):
    class Meta:
        model = Notification
        fields = [
            "id",
            "sender",
            "recipient",
            "title",
            "description",
            "link",
            "icon",
            "send_at",
            "read",
            "sent",
            "created",
            "modified",
        ]

    @staticmethod
    def resolve_recipient(root, info, **kwargs):
        if info.context.user.has_perm("core.view_person_rule", root.recipient):
            return root.recipient
        raise PermissionDenied()


class MarkNotificationReadMutation(graphene.Mutation):
    class Arguments:
        id = graphene.ID()  # noqa

    notification = graphene.Field(NotificationType)

    @classmethod
    def mutate(cls, root, info, id):  # noqa
        notification = Notification.objects.get(pk=id)

        if not info.context.user.has_perm("core.mark_notification_as_read_rule", notification):
            raise PermissionDenied()
        notification.read = True
        notification.save()

        return MarkNotificationReadMutation(notification=notification)



# Generated by Django 3.1.3 on 2020-11-16 13:04

from django.db import migrations, models
import django.utils.timezone
import model_utils.fields


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0004_add_permissions_for_group_stats'),
    ]

    operations = [
        migrations.AddField(
            model_name='activity',
            name='created',
            field=model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created'),
        ),
        migrations.AddField(
            model_name='activity',
            name='modified',
            field=model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified'),
        ),
        migrations.AddField(
            model_name='notification',
            name='created',
            field=model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created'),
        ),
        migrations.AddField(
            model_name='notification',
            name='modified',
            field=model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified'),
        ),
    ]



from datetime import date
from typing import Union

from django.apps import apps
from django.contrib.sites.managers import CurrentSiteManager as _CurrentSiteManager
from django.db.models import QuerySet
from django.db.models.manager import Manager

from calendarweek import CalendarWeek
from django_cte import CTEManager, CTEQuerySet
from polymorphic.managers import PolymorphicManager


class CurrentSiteManagerWithoutMigrations(_CurrentSiteManager):
    """CurrentSiteManager for auto-generating managers just by query sets."""

    use_in_migrations = False


class DateRangeQuerySetMixin:
    """QuerySet with custom query methods for models with date ranges.

    Filterable fields: date_start, date_end
    """

    def within_dates(self, start: date, end: date):
        """Filter for all objects within a date range."""
        return self.filter(date_start__lte=end, date_end__gte=start)

    def in_week(self, wanted_week: CalendarWeek):
        """Filter for all objects within a calendar week."""
        return self.within_dates(wanted_week[0], wanted_week[6])

    def on_day(self, day: date):
        """Filter for all objects on a certain day."""
        return self.within_dates(day, day)


class SchoolTermQuerySet(QuerySet, DateRangeQuerySetMixin):
    """Custom query set for school terms."""


class SchoolTermRelatedQuerySet(QuerySet):
    """Custom query set for all models related to school terms."""

    def within_dates(self, start: date, end: date) -> "SchoolTermRelatedQuerySet":
        """Filter for all objects within a date range."""
        return self.filter(school_term__date_start__lte=end, school_term__date_end__gte=start)

    def in_week(self, wanted_week: CalendarWeek) -> "SchoolTermRelatedQuerySet":
        """Filter for all objects within a calendar week."""
        return self.within_dates(wanted_week[0], wanted_week[6])

    def on_day(self, day: date) -> "SchoolTermRelatedQuerySet":
        """Filter for all objects on a certain day."""
        return self.within_dates(day, day)

    def for_school_term(self, school_term: "SchoolTerm") -> "SchoolTermRelatedQuerySet":
        return self.filter(school_term=school_term)

    def for_current_school_term_or_all(self) -> "SchoolTermRelatedQuerySet":
        """Get all objects related to current school term.

        If there is no current school term, it will return all objects.
        """
        from aleksis.core.models import SchoolTerm

        current_school_term = SchoolTerm.current
        if current_school_term:
            return self.for_school_term(current_school_term)
        else:
            return self

    def for_current_school_term_or_none(self) -> Union["SchoolTermRelatedQuerySet", None]:
        """Get all objects related to current school term.

        If there is no current school term, it will return `None`.
        """
        from aleksis.core.models import SchoolTerm

        current_school_term = SchoolTerm.current
        if current_school_term:
            return self.for_school_term(current_school_term)
        else:
            return None


class GroupManager(CurrentSiteManagerWithoutMigrations, CTEManager):
    """Manager adding specific methods to groups."""

    def get_queryset(self):
        """Ensure all related data is loaded as well."""
        return Manager.get_queryset(self).select_related("school_term")


class GroupQuerySet(SchoolTermRelatedQuerySet, CTEQuerySet):
    pass


class UninstallRenitentPolymorphicManager(PolymorphicManager):
    """A custom manager for django-polymorphic that filters out submodels of unavailable apps."""

    def get_queryset(self):
        DashboardWidget = apps.get_model("core", "DashboardWidget")
        if self.model is DashboardWidget:
            return super().get_queryset().instance_of(*self.model.__subclasses__())
        else:
            # Called on subclasses
            return super().get_queryset()


class InstalledWidgetsDashboardWidgetOrderManager(Manager):
    """A manager that only returns DashboardWidgetOrder objects with an existing widget."""

    def get_queryset(self):
        queryset = super().get_queryset()

        # Get the DashboardWidget model class without importing it to avoid a circular import
        DashboardWidget = queryset.model.widget.field.related_model  # noqa
        dashboard_widget_pks = DashboardWidget.objects.all().values("id")

        # [obj["id"] for obj in list(Person.objects.all().values("id"))]
        return super().get_queryset().filter(widget_id__in=dashboard_widget_pks)


class PolymorphicCurrentSiteManager(_CurrentSiteManager, PolymorphicManager):
    """Default manager for extensible, polymorphic models."""



import json
import os
import shutil
from typing import Any, Optional, Sequence

from django.conf import settings

from django_yarnpkg.yarn import yarn_adapter

from .core_helpers import get_app_module, get_app_packages


def get_apps_with_frontend():
    """Get a dictionary of apps that ship frontend code/assets."""
    assets = {}
    for app in get_app_packages():
        mod = get_app_module(app, "apps")
        path = os.path.join(os.path.dirname(mod.__file__), "frontend")
        if os.path.isdir(path):
            package = ".".join(app.split(".")[:-2])
            assets[package] = path
    return assets


def write_vite_values(out_path: str) -> dict[str, Any]:
    vite_values = {
        "static_url": settings.STATIC_URL,
        "serverPort": settings.DJANGO_VITE_DEV_SERVER_PORT,
    }
    # Write rollup entrypoints for all apps
    vite_values["appDetails"] = {}
    for app, path in get_apps_with_frontend().items():
        if os.path.exists(path):
            vite_values["appDetails"][app] = {}
            vite_values["appDetails"][app]["name"] = app.split(".")[-1]
            vite_values["appDetails"][app]["assetDir"] = path
            vite_values["appDetails"][app]["hasMessages"] = os.path.exists(
                os.path.join(path, "messages", "en.json")
            )
    # Add core entrypoint
    vite_values["coreAssetDir"] = os.path.join(settings.BASE_DIR, "aleksis", "core", "frontend")

    # Add directories
    vite_values["baseDir"] = settings.BASE_DIR
    vite_values["cacheDir"] = settings.CACHE_DIR
    vite_values["node_modules"] = settings.JS_ROOT

    with open(out_path, "w") as out:
        json.dump(vite_values, out)


def run_vite(args: Optional[Sequence[str]] = None) -> int:
    args = list(args) if args else []

    config_path = os.path.join(settings.BASE_DIR, "aleksis", "core", "vite.config.js")
    shutil.copy(config_path, settings.NODE_MODULES_ROOT)

    mode = "development" if settings.DEBUG else "production"
    args += ["-m", mode]

    log_level = settings.LOGGING["root"]["level"]
    if settings.DEBUG or log_level == "DEBUG":
        args.append("-d")
    log_level = {"INFO": "info", "WARNING": "warn", "ERROR": "error"}.get(log_level, "silent")
    args += ["-l", log_level]

    return yarn_adapter.call_yarn(["run", "vite"] + args)


def get_language_cookie(code: str) -> str:
    """Build a cookie string to set a new language."""
    cookie_parts = [f"{settings.LANGUAGE_COOKIE_NAME}={code}"]
    args = dict(
        max_age=settings.LANGUAGE_COOKIE_AGE,
        path=settings.LANGUAGE_COOKIE_PATH,
        domain=settings.LANGUAGE_COOKIE_DOMAIN,
        secure=settings.LANGUAGE_COOKIE_SECURE,
        httponly=settings.LANGUAGE_COOKIE_HTTPONLY,
        samesite=settings.LANGUAGE_COOKIE_SAMESITE,
    )
    cookie_parts += [f"{k.replace('_', '-')}={v}" for k, v in args.items() if v]
    return "; ".join(cookie_parts)



from typing import Optional

import django.apps
from django.core.checks import Tags, Warning, register  # noqa: A004

from .mixins import ExtensibleModel, GlobalPermissionModel, PureDjangoModel
from .util.apps import AppConfig


@register(Tags.compatibility)
def check_app_configs_base_class(
    app_configs: Optional[django.apps.registry.Apps] = None, **kwargs
) -> list:
    """Check whether all apps derive from AlekSIS's base app config."""
    results = []

    if app_configs is None:
        app_configs = django.apps.apps.get_app_configs()

    for app_config in filter(lambda c: c.name.startswith("aleksis."), app_configs):
        if not isinstance(app_config, AppConfig):
            results.append(
                Warning(
                    f"App config {app_config.name} does not derive"
                    "from aleksis.core.util.apps.AppConfig.",
                    hint=(
                        "Ensure the app uses the correct base class for all"
                        "registry functionality to work."
                    ),
                    obj=app_config,
                    id="aleksis.core.W001",
                )
            )

    return results


@register(Tags.compatibility)
def check_app_models_base_class(
    app_configs: Optional[django.apps.registry.Apps] = None, **kwargs
) -> list:
    """Check whether all app models derive from AlekSIS's allowed base models.

    Does only allow ExtensibleModel, GlobalPermissionModel and PureDjangoModel.
    """
    results = []

    if app_configs is None:
        app_configs = django.apps.apps.get_app_configs()

    for app_config in filter(lambda c: c.name.startswith("aleksis."), app_configs):
        for model in app_config.get_models():
            if not (
                set(model.__mro__) & set((ExtensibleModel, PureDjangoModel, GlobalPermissionModel))
            ):
                results.append(
                    Warning(
                        f"Model {model._meta.object_name} in app config {app_config.name} does "
                        "not derive from aleksis.core.mixins.ExtensibleModel "
                        "or aleksis.core.mixins.GlobalPermissionModel.",
                        hint=(
                            "Ensure all models in AlekSIS use ExtensibleModel (or "
                            "GlobalPermissionModel, if you want to define global permissions) "
                            "as base. "
                            "If your deviation is intentional, you can add the PureDjangoModel "
                            "mixin instead to silence this warning."
                        ),
                        obj=model,
                        id="aleksis.core.W002",
                    )
                )

    return results



import pytest

from aleksis.core.models import Person

pytestmark = pytest.mark.django_db


def test_full_name():
    _person = Person.objects.create(first_name="Jane", last_name="Doe")

    assert _person.full_name == "Doe, Jane"


def test_delete():
    _person = Person.objects.create(first_name="Jane", last_name="Doe")
    _person.delete()
    assert not Person.objects.filter(first_name="Jane", last_name="Doe").exists()



"""Helpers/overrides for django-allauth."""

from typing import Any, Optional

from django.conf import settings
from django.contrib.auth.validators import ASCIIUsernameValidator
from django.core.validators import RegexValidator
from django.http import HttpRequest

from allauth.account.adapter import DefaultAccountAdapter
from allauth.socialaccount.adapter import DefaultSocialAccountAdapter
from oauth2_provider.models import AbstractApplication
from oauth2_provider.oauth2_validators import OAuth2Validator
from oauth2_provider.scopes import BaseScopes
from oauth2_provider.views.mixins import (
    ClientProtectedResourceMixin as _ClientProtectedResourceMixin,
)
from oauthlib.common import Request as OauthlibRequest

from .apps import AppConfig
from .core_helpers import get_site_preferences


class OurSocialAccountAdapter(DefaultSocialAccountAdapter):
    """Customised adapter that recognises other authentication mechanisms."""

    def validate_disconnect(self, account, accounts):
        """Validate whether or not the socialaccount account can be safely disconnected.

        Honours other authentication backends, i.e. ignores unusable passwords if LDAP is used.
        """
        if "django_auth_ldap.backend.LDAPBackend" in settings.AUTHENTICATION_BACKENDS:
            # Ignore upstream validation error as we do not need a usable password
            return None

        # Let upstream decide whether we can disconnect or not
        return super().validate_disconnect(account, accounts)


class OurAccountAdapter(DefaultAccountAdapter):
    """Customised adapter to allow to disable signup."""

    def is_open_for_signup(self, request):
        return get_site_preferences()["auth__signup_enabled"]


class CustomOAuth2Validator(OAuth2Validator):
    def get_additional_claims(self, request: OauthlibRequest) -> dict[str, Any]:
        # Pull together scopes from request and from access token
        scopes = request.scopes.copy()
        if request.access_token:
            scopes += request.access_token.scope.split(" ")

        claims = {}
        # Pull together claim data from all apps
        for app in AppConfig.__subclasses__():
            claims.update(app.get_additional_claims(scopes, request))

        return claims

    def get_userinfo_claims(self, request: OauthlibRequest) -> dict[str, Any]:
        oidc_claims = super().get_oidc_claims(request.access_token, None, request)
        additional_claims = self.get_additional_claims(request)
        return oidc_claims | additional_claims


class AppScopes(BaseScopes):
    """Scopes backend for django-oauth-toolkit gathering scopes from apps.

    Will call the respective method on all known AlekSIS app configs and
    join the results.
    """

    def get_all_scopes(self) -> dict[str, str]:
        scopes = {}
        for app in AppConfig.__subclasses__():
            scopes |= app.get_all_scopes()
        return scopes

    def get_available_scopes(
        self,
        application: Optional[AbstractApplication] = None,
        request: Optional[HttpRequest] = None,
        *args,
        **kwargs
    ) -> list[str]:
        scopes = []
        for app in AppConfig.__subclasses__():
            scopes += app.get_available_scopes()
        # Filter by allowed scopes of requesting application
        if application and application.allowed_scopes:
            scopes = list(filter(lambda scope: scope in application.allowed_scopes, scopes))
        return scopes

    def get_default_scopes(
        self,
        application: Optional[AbstractApplication] = None,
        request: Optional[HttpRequest] = None,
        *args,
        **kwargs
    ) -> list[str]:
        scopes = []
        for app in AppConfig.__subclasses__():
            scopes += app.get_default_scopes()
        # Filter by allowed scopes of requesting application
        if application and application.allowed_scopes:
            scopes = list(filter(lambda scope: scope in application.allowed_scopes, scopes))
        return scopes


class ClientProtectedResourceMixin(_ClientProtectedResourceMixin):
    """Mixin for protecting resources with client authentication as mentioned in rfc:`3.2.1`.

    This involves authenticating with any of: HTTP Basic Auth, Client Credentials and
    Access token in that order. Breaks off after first validation.

    This sub-class extends the functionality of Django OAuth Toolkit's mixin with support
    for AlekSIS's `allowed_scopes` feature. For applications that have configured allowed
    scopes, the required scopes for the view are checked to be a subset of the application's
    allowed scopes (best to be combined with ScopedResourceMixin).
    """

    def authenticate_client(self, request: HttpRequest) -> bool:
        """Return a boolean representing if client is authenticated with client credentials.

        If the view has configured required scopes, they are verified against the application's
        allowed scopes.
        """
        # Build an OAuth request so we can handle client information
        core = self.get_oauthlib_core()
        uri, http_method, body, headers = core._extract_params(request)
        oauth_request = OauthlibRequest(uri, http_method, body, headers)

        # Verify general authentication of the client
        if not core.server.request_validator.authenticate_client(oauth_request):
            # Client credentials were invalid
            return False

        # Verify scopes of configured application
        # The OAuth request was enriched with a reference to the Application when using the
        #  validator above.
        if not oauth_request.client.allowed_scopes:
            # If there are no allowed scopes, the client is not allowed to access this resource
            return False

        required_scopes = set(self.get_scopes() or [])
        allowed_scopes = set(AppScopes().get_available_scopes(oauth_request.client) or [])
        return required_scopes.issubset(allowed_scopes)


def validate_username_preference_regex(value: str):
    regex = get_site_preferences()["auth__allowed_username_regex"]
    return RegexValidator(regex)(value)


custom_username_validators = [validate_username_preference_regex, ASCIIUsernameValidator()]



from dj_iconify.util import icon_choices

# Materialize colors (without accent, darken and lighten classes)
COLOURS = [
    ("red", "red"),
    ("pink", "pink"),
    ("purple", "purple"),
    ("deep-purple", "deep-purple"),
    ("indigo", "indigo"),
    ("blue", "blue"),
    ("light-blue", "light-blue"),
    ("cyan", "cyan"),
    ("teal", "teal"),
    ("green", "green"),
    ("light-green", "light-green"),
    ("lime", "lime"),
    ("yellow", "yellow"),
    ("amber", "amber"),
    ("orange", "orange"),
    ("deep-orange", "deep-orange"),
    ("brown", "brown"),
    ("grey", "grey"),
    ("blue-grey", "blue-grey"),
    ("black", "black"),
    ("white", "white"),
    ("transparent", "transparent"),
]

try:
    ICONS = icon_choices("mdi")
except FileNotFoundError:
    # If icons aren't installed yet, set choices to empty list
    ICONS = []



from django.core.exceptions import PermissionDenied

import graphene
from graphene_django import DjangoObjectType

from ..models import PDFFile
from .base import FieldFileType


class PDFFileType(DjangoObjectType):
    file = graphene.Field(FieldFileType)

    class Meta:
        model = PDFFile
        exclude = ["html_file"]

    @staticmethod
    def resolve_person(root, info, **kwargs):
        if info.context.user.has_perm("core.view_person_rule", root.person):
            return root.person
        raise PermissionDenied()



# Generated by Django 3.2.9 on 2021-12-09 14:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0032_remove_person_is_active'),
    ]

    operations = [
        migrations.AddField(
            model_name='group',
            name='avatar',
            field=models.ImageField(blank=True, help_text='This is a picture or an avatar for public display.', null=True, upload_to='', verbose_name='Display picture / Avatar'),
        ),
        migrations.AddField(
            model_name='group',
            name='photo',
            field=models.ImageField(blank=True, help_text='This is an official photo, used for official documents and for internal use cases.', null=True, upload_to='', verbose_name='Photo'),
        ),
        migrations.AddField(
            model_name='person',
            name='avatar',
            field=models.ImageField(blank=True, help_text='This is a picture or an avatar for public display.', null=True, upload_to='', verbose_name='Display picture / Avatar'),
        ),
        migrations.AlterField(
            model_name='person',
            name='photo',
            field=models.ImageField(blank=True, help_text='This is an official photo, used for official documents and for internal use cases.', null=True, upload_to='', verbose_name='Photo'),
        ),
        migrations.AlterModelOptions(
            name='globalpermissions',
            options={'default_permissions': (), 'managed': False, 'permissions': (('view_system_status', 'Can view system status'), ('manage_data', 'Can manage data'), ('impersonate', 'Can impersonate'), ('search', 'Can use search'), ('change_site_preferences', 'Can change site preferences'), ('change_person_preferences', 'Can change person preferences'), ('change_group_preferences', 'Can change group preferences'), ('test_pdf', 'Can test PDF generation'))},
        ),
        migrations.AlterModelOptions(
            name='person',
            options={'ordering': ['last_name', 'first_name'], 'permissions': (('view_address', 'Can view address'), ('view_contact_details', 'Can view contact details'), ('view_photo', 'Can view photo'), ('view_avatar', 'Can view avatar image'), ('view_person_groups', 'Can view persons groups'), ('view_personal_details', 'Can view personal details')), 'verbose_name': 'Person', 'verbose_name_plural': 'Persons'},
        ),
    ]



# Generated by Django 3.2.12 on 2022-02-20 21:04

from django.db import migrations, models
import django.db.models.deletion
import uuid



class Migration(migrations.Migration):

    dependencies = [
        ('core', '0038_notification_send_at'),
    ]

    operations = [
        migrations.CreateModel(
            name='PersonalICalUrl',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False, unique=True, verbose_name='UUID')),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('person', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='calendar_urls', to='core.person', verbose_name='Person')),
            ],
            options={
                'verbose_name': 'Personal Calendar URL',
                'verbose_name_plural': 'Personal Calendar URLs',
            },
        ),
    ]



import graphene


class MessageType(graphene.ObjectType):
    tags = graphene.String()
    message = graphene.String()

    def resolve_tags(root, info, **kwargs):
        return root.tags

    def resolve_message(root, info, **kwargs):
        return root.message



import os

from django.conf import settings
from django.core.management.base import CommandError

from django_yarnpkg.management.base import BaseYarnCommand
from django_yarnpkg.yarn import yarn_adapter

from ...util.frontend_helpers import run_vite, write_vite_values


class Command(BaseYarnCommand):
    help = "Create Vite bundles for AlekSIS"  # noqa

    def add_arguments(self, parser):
        parser.add_argument("command", choices=["build", "serve"], nargs="?", default="build")
        parser.add_argument("--no-install", action="store_true", default=False)

    def handle(self, *args, **options):
        super(Command, self).handle(*args, **options)

        # Inject settings into Vite
        write_vite_values(os.path.join(settings.NODE_MODULES_ROOT, "django-vite-values.json"))

        # Install Node dependencies
        if not options["no_install"]:
            yarn_adapter.install(settings.YARN_INSTALLED_APPS)

        # Run Vite build
        ret = run_vite([options["command"]])
        if ret != 0:
            raise CommandError("yarn command failed", returncode=ret)



# Generated by Django 3.2.12 on 2022-02-23 19:33

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0037_add_static_content_widget'),
    ]

    operations = [
        migrations.AddField(
            model_name='notification',
            name='send_at',
            field=models.DateTimeField(default=django.utils.timezone.now, verbose_name='Send notification at'),
            preserve_default=False,
        ),
    ]



# Generated by Django 3.2.11 on 2022-01-22 20:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0033_update_photo_avatar'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='globalpermissions',
            options={'default_permissions': (), 'managed': False, 'permissions': (('view_system_status', 'Can view system status'), ('manage_data', 'Can manage data'), ('impersonate', 'Can impersonate'), ('search', 'Can use search'), ('change_site_preferences', 'Can change site preferences'), ('change_person_preferences', 'Can change person preferences'), ('change_group_preferences', 'Can change group preferences'), ('test_pdf', 'Can test PDF generation'), ('invite', 'Can invite persons'))},
        ),
    ]



from django.apps import apps
from django.contrib.messages import get_messages
from django.core.exceptions import PermissionDenied
from django.db.models import Q

import graphene
from guardian.shortcuts import get_objects_for_user
from haystack.inputs import AutoQuery
from haystack.query import SearchQuerySet
from haystack.utils.loading import UnifiedIndex

from ..models import (
    CustomMenu,
    DynamicRoute,
    Notification,
    OAuthAccessToken,
    PDFFile,
    Person,
    TaskUserAssignment,
)
from ..util.apps import AppConfig
from ..util.core_helpers import get_allowed_object_ids, get_app_module, get_app_packages, has_person
from .celery_progress import CeleryProgressFetchedMutation, CeleryProgressType
from .custom_menu import CustomMenuType
from .dynamic_routes import DynamicRouteType
from .group import GroupType  # noqa
from .installed_apps import AppType
from .message import MessageType
from .notification import MarkNotificationReadMutation, NotificationType
from .oauth import OAuthAccessTokenType, OAuthRevokeTokenMutation
from .pdf import PDFFileType
from .person import PersonMutation, PersonType
from .school_term import SchoolTermType  # noqa
from .search import SearchResultType
from .system_properties import SystemPropertiesType
from .two_factor import TwoFactorType
from .user import UserType


class Query(graphene.ObjectType):
    ping = graphene.String(payload=graphene.String())

    notifications = graphene.List(NotificationType)

    persons = graphene.List(PersonType)
    person_by_id = graphene.Field(PersonType, id=graphene.ID())
    person_by_id_or_me = graphene.Field(PersonType, id=graphene.ID())

    who_am_i = graphene.Field(UserType)

    system_properties = graphene.Field(SystemPropertiesType)
    installed_apps = graphene.List(AppType)

    celery_progress_by_task_id = graphene.Field(CeleryProgressType, task_id=graphene.String())
    celery_progress_by_user = graphene.List(CeleryProgressType)

    pdf_by_id = graphene.Field(PDFFileType, id=graphene.ID())

    search_snippets = graphene.List(
        SearchResultType, query=graphene.String(), limit=graphene.Int(required=False)
    )

    messages = graphene.List(MessageType)

    custom_menu_by_name = graphene.Field(CustomMenuType, name=graphene.String())

    dynamic_routes = graphene.List(DynamicRouteType)

    two_factor = graphene.Field(TwoFactorType)

    oauth_access_tokens = graphene.List(OAuthAccessTokenType)

    def resolve_ping(root, info, payload) -> str:
        return payload

    def resolve_notifications(root, info, **kwargs):
        return Notification.objects.filter(
            Q(
                pk__in=get_objects_for_user(
                    info.context.user, "core.view_person", Person.objects.all()
                )
            )
            | Q(recipient=info.context.user.person)
        )

    def resolve_persons(root, info, **kwargs):
        return get_objects_for_user(info.context.user, "core.view_person", Person.objects.all())

    def resolve_person_by_id(root, info, id):  # noqa
        person = Person.objects.get(pk=id)
        if not info.context.user.has_perm("core.view_person_rule", person):
            raise PermissionDenied()
        return person

    def resolve_person_by_id_or_me(root, info, **kwargs):  # noqa
        # Returns person associated with current user if id is None, else the person with the id
        if "id" not in kwargs or kwargs["id"] is None:
            return info.context.user.person if has_person(info.context.user) else None

        person = Person.objects.get(pk=kwargs["id"])
        if not info.context.user.has_perm("core.view_person_rule", person):
            raise PermissionDenied()
        return person

    def resolve_who_am_i(root, info, **kwargs):
        return info.context.user

    def resolve_system_properties(root, info, **kwargs):
        return True

    def resolve_installed_apps(root, info, **kwargs):
        return [app for app in apps.get_app_configs() if isinstance(app, AppConfig)]

    def resolve_celery_progress_by_task_id(root, info, task_id, **kwargs):
        task = TaskUserAssignment.objects.get(task_result__task_id=task_id)

        if not info.context.user.has_perm("core.view_progress_rule", task):
            raise PermissionDenied()
        progress = task.get_progress_with_meta()
        return progress

    def resolve_celery_progress_by_user(root, info, **kwargs):
        if info.context.user.is_anonymous:
            return None
        tasks = TaskUserAssignment.objects.filter(user=info.context.user)
        return [
            task.get_progress_with_meta()
            for task in tasks
            if task.get_progress_with_meta()["complete"] is False
        ]

    def resolve_pdf_by_id(root, info, id, **kwargs):  # noqa
        pdf_file = PDFFile.objects.get(pk=id)
        if has_person(info.context) and info.context.user.person == pdf_file.person:
            return pdf_file
        return None

    def resolve_search_snippets(root, info, query, limit=-1, **kwargs):
        indexed_models = UnifiedIndex().get_indexed_models()
        allowed_object_ids = get_allowed_object_ids(info.context.user, indexed_models)

        if allowed_object_ids:
            results = (
                SearchQuerySet().filter(id__in=allowed_object_ids).filter(text=AutoQuery(query))
            )
            if limit < 0:
                return results
            return results[:limit]
        else:
            return None

    def resolve_messages(root, info, **kwargs):
        return get_messages(info.context)

    def resolve_custom_menu_by_name(root, info, name, **kwargs):
        return CustomMenu.get_default(name)

    def resolve_dynamic_routes(root, info, **kwargs):
        dynamic_routes = []

        for dynamic_route_object in DynamicRoute.registered_objects_dict.values():
            dynamic_routes += dynamic_route_object.get_dynamic_routes()

        return dynamic_routes

    def resolve_two_factor(root, info, **kwargs):
        if info.context.user.is_anonymous:
            return None
        return info.context.user

    @staticmethod
    def resolve_oauth_access_tokens(root, info, **kwargs):
        return OAuthAccessToken.objects.filter(user=info.context.user)


class Mutation(graphene.ObjectType):
    update_person = PersonMutation.Field()

    mark_notification_read = MarkNotificationReadMutation.Field()

    celery_progress_fetched = CeleryProgressFetchedMutation.Field()

    revoke_oauth_token = OAuthRevokeTokenMutation.Field()


def build_global_schema():
    """Build global GraphQL schema from all apps."""
    query_bases = [Query]
    mutation_bases = [Mutation]

    for app in get_app_packages():
        schema_mod = get_app_module(app, "schema")
        if not schema_mod:
            # The app does not define a schema
            continue

        if AppQuery := getattr(schema_mod, "Query", None):
            query_bases.append(AppQuery)
        if AppMutation := getattr(schema_mod, "Mutation", None):
            mutation_bases.append(AppMutation)

    # Define classes using all query/mutation classes as mixins
    #  cf. https://docs.graphene-python.org/projects/django/en/latest/schema/#adding-to-the-schema
    GlobalQuery = type("GlobalQuery", tuple(query_bases), {})
    GlobalMutation = type("GlobalMutation", tuple(mutation_bases), {})

    return graphene.Schema(query=GlobalQuery, mutation=GlobalMutation)


schema = build_global_schema()



# Generated by Django 3.1.7 on 2021-03-31 16:55

import aleksis.core.mixins
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('core', '0028_char_field_not_null'),
    ]

    operations = [
        migrations.CreateModel(
            name='PersonInvitation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('accepted', models.BooleanField(default=False, verbose_name='accepted')),
                ('key', models.CharField(max_length=64, unique=True, verbose_name='key')),
                ('sent', models.DateTimeField(null=True, verbose_name='sent')),
                ('email', models.EmailField(blank=True, max_length=254, verbose_name='E-Mail address')),
                ('inviter', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
                ('person', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='invitation', to='core.person'))
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model, aleksis.core.mixins.PureDjangoModel),
        ),
    ]



import graphene

from .permissions import GlobalPermissionType
from .person import PersonType


class UserType(graphene.ObjectType):
    id = graphene.ID()  # noqa
    username = graphene.String()
    first_name = graphene.String()
    last_name = graphene.String()

    is_authenticated = graphene.Boolean(required=True)
    is_anonymous = graphene.Boolean(required=True)
    is_impersonate = graphene.Boolean()

    person = graphene.Field(PersonType)

    global_permissions_by_name = graphene.List(
        GlobalPermissionType, permissions=graphene.List(graphene.String)
    )

    def resolve_global_permissions_by_name(root, info, permissions, **kwargs):
        return [
            {"name": permission_name, "result": info.context.user.has_perm(permission_name)}
            for permission_name in permissions
        ]



from re import match, sub

from django.apps import apps
from django.core.management.base import BaseCommand, CommandError

from aleksis.core.util.core_helpers import get_app_module


def camelcase(value: str) -> str:
    """Convert a string to camelcase."""
    titled = value.replace("_", " ").title().replace(" ", "")
    return titled[0].lower() + titled[1:]


class Command(BaseCommand):
    help = "Convert Django URLs for an app into vue-router routes"  # noqa

    def add_arguments(self, parser):
        parser.add_argument("app", type=str)

    def handle(self, *args, **options):
        app = options["app"]
        app_camel_case = camelcase(app)

        app_config = apps.get_app_config(app)
        app_config_name = f"{app_config.__module__}.{app_config.__class__.__name__}"

        # Import urls from app
        urls = get_app_module(app_config_name, "urls")
        if not urls:
            raise CommandError(f"No url patterns found in app {app}")
        urlpatterns = urls.urlpatterns

        # Import menu from app and structure as dict by url name
        menus = get_app_module(app_config_name, "menus")
        menu_by_urls = {}
        if "NAV_MENU_CORE" in menus.MENUS:
            menu = menus.MENUS["NAV_MENU_CORE"]
            menu_by_urls = {m["url"]: m for m in menu}

            for menu_item in menu:
                if "submenu" in menu_item:
                    for submenu_item in menu_item["submenu"]:
                        menu_by_urls[submenu_item["url"]] = submenu_item

        for url in urlpatterns:
            # Convert route name and url pattern to vue-router format
            menu = menu_by_urls[url.name] if url.name in menu_by_urls else None
            route_name = f"{app_camel_case}.{camelcase(url.name)}"
            url_pattern = url.pattern._route
            new_url_pattern_list = []
            for url_pattern_part in url_pattern.split("/"):
                if match(r"<[\w,:,*]*>", url_pattern_part):
                    url_pattern_part = sub(r"(<(?P<val>[\w,:,*]*)>)", r":\g<val>", url_pattern_part)
                    new_url_pattern_list.append(":" + url_pattern_part.split(":")[-1])
                else:
                    new_url_pattern_list.append(url_pattern_part)
            url_pattern = "/".join(new_url_pattern_list)

            # Start building route
            route = "{\n"
            route += f'  path: "{url_pattern}",\n'
            route += (
                '  component: () => import("aleksis.core/components/LegacyBaseTemplate.vue"),\n'
            )
            route += f'  name: "{route_name}",\n'

            if menu:
                # Convert icon to Vuetify format
                icon = None
                if menu.get("vuetify_icon"):
                    icon = menu["vuetify_icon"]
                elif menu.get("svg_icon"):
                    icon = menu["svg_icon"].replace(":", "-")
                elif menu.get("icon"):
                    icon = "mdi-" + menu["icon"]

                if icon:
                    icon = icon.replace("_", "-")

                # Get permission for menu item
                permission = None
                if menu.get("validators"):
                    possible_validators = [
                        v
                        for v in menu["validators"]
                        if v[0] == "aleksis.core.util.predicates.permission_validator"
                    ]
                    if possible_validators:
                        permission = possible_validators[0][1]

                route += "  meta: {\n"
                route += "    inMenu: true,\n"
                route += f'    titleKey: "{menu["name"]}", // Needs manual work\n'
                if icon:
                    route += f'    icon: "{icon}",\n'
                if permission:
                    route += f'    permission: "{permission}",\n'
                route += "  },\n"
            route += "},"

            print(route)



# Generated by Django 3.2.16 on 2022-11-02 19:35

import django.utils.timezone
from django.db import migrations, models

import oauth2_provider.generators
import oauth2_provider.models


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0043_task_assignment_meta"),
    ]

    operations = [
        migrations.AddField(
            model_name="taskuserassignment",
            name="result_fetched",
            field=models.BooleanField(default=False, verbose_name="Result fetched"),
        ),
    ]



# Generated by Django 4.2.13 on 2024-05-22 08:49

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('core', '0049_oauthapplication_post_logout_redirect_uris'),
    ]

    operations = [
        migrations.AddField(
            model_name='oauthapplication',
            name='allowed_origins',
            field=models.TextField(blank=True, default='', help_text='Allowed origins list to enable CORS, space separated'),
        ),
        migrations.AddField(
            model_name='oauthapplication',
            name='hash_client_secret',
            field=models.BooleanField(default=True),
        ),
        migrations.AlterField(
            model_name='oauthaccesstoken',
            name='token',
            field=models.CharField(db_index=True, max_length=255, unique=True),
        ),
        migrations.AlterField(
            model_name='oauthapplication',
            name='post_logout_redirect_uris',
            field=models.TextField(blank=True, default='', help_text='Allowed Post Logout URIs list, space separated'),
        ),
    ]
